import os
import yaml

from logging import getLogger

logger = getLogger(__name__)


class MockAppEngineEnvironment:
    @classmethod
    def load_app_yaml(cls, app_yaml_path):
        with open(app_yaml_path, 'r+') as f:
            app_yaml = yaml.load(f, Loader=yaml.CLoader)

        if 'env_variables' in app_yaml:
            for key, value in app_yaml['env_variables'].items():
                os.environ[key] = value



from gumo.core._configuration import configure

from gumo.core.exceptions import ConfigurationError

from gumo.core.domain.configuration import GumoConfiguration
from gumo.core.domain.configuration import GoogleCloudLocation
from gumo.core.domain.configuration import GoogleCloudProjectID

from gumo.core.domain.entity_key import EntityKey
from gumo.core.domain.entity_key import NoneKey
from gumo.core.domain.entity_key import EntityKeyFactory

from gumo.core.infrastructure import MockAppEngineEnvironment
from gumo.core.infrastructure.credential import get_credential


__all__ = [
    configure.__name__,

    ConfigurationError.__name__,

    GumoConfiguration.__name__,
    GoogleCloudLocation.__name__,
    GoogleCloudProjectID.__name__,

    EntityKey.__name__,
    NoneKey.__name__,
    EntityKeyFactory.__name__,

    MockAppEngineEnvironment.__name__,
    get_credential.__name__,
]



import json
import google.oauth2
from google.cloud import storage

from gumo.core.injector import injector
from gumo.core.domain.configuration import GumoConfiguration


class CredentialManager:
    _credential = None

    @classmethod
    def get_credential(cls):
        if cls._credential:
            return cls._credential

        cls._credential = injector.get(cls).build_credential_from_storage()
        return cls._credential

    @injector
    def __init__(
            self,
            gumo_configuration: GumoConfiguration,
    ):
        self._gumo_configuration = gumo_configuration
        self._credential_config = self._gumo_configuration.service_account_credential_config

    def build_credential_from_storage(self):
        if not self._credential_config.enabled:
            raise RuntimeError(f'ServiceAccount Credential Config disabled.')

        return google.oauth2.service_account.Credentials.from_service_account_info(
            info=self._get_content()
        )

    def _get_content(self):
        storage_client = storage.Client()
        bucket = storage_client.get_bucket(bucket_name=self._credential_config.bucket_name)
        blob = bucket.blob(blob_name=self._credential_config.blob_path)
        content = blob.download_as_string(client=storage_client)

        return json.loads(content)


def get_credential():
    return CredentialManager.get_credential()
